#  This program may be used, executed, copied, modified and distributed
#  without royalty for the purpose of developing, using, marketing, or distributing.
#
# defaultWorkClasses  - creates default work classes
#
# @author - mjsmith
#
# Change History
# --------------
# 07-06-2005 initial version created
# 04-03-2006 cleaned up for maintainability purposes.
#=======================================================================================================================================================

import sys
import com.ibm.ws.sm.workspace.WorkSpace as WorkSpace
import com.ibm.ws.xd.config.workclass.util.ApplicationUtil as ApplicationUtil
import com.ibm.ws.xd.operationalpolicymonitor.util.MonitorUtil as MonitorUtil

lineSeparator = java.lang.System.getProperty('line.separator')


def createAppRoutingWorkClass(modules, protocol):
   routingWorkClass=""
   appLongName=findDeployment(appname)
   appShortName=appname

   if (appLongName != None):
       routingWorkClass = findAppRoutingWorkClass(getWorkClassName(protocol))
       if (routingWorkClass == None):
         taskCmdArgs =  '[-appname "'+ appShortName+'" -wcname '+getWorkClassName(protocol)+' -type '+protocol+' -actiontype permit -action "'+appname+'"]'
         routingWorkClass = AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
         print ("Created... default routing workclass: "+getWorkClassName(protocol))
         
       if (routingWorkClass != None):
          createDefaultModules(routingWorkClass, modules, protocol)
          
   else:
      print "The application specified does not exist in this WAS deployment"
      errorExit()

    
def createAppSLAWorkClass(modules, protocol):
    slaWorkClass=""
    appLongName=findDeployment(appname)
    appShortName=appname
       
    if (appLongName != None):
        slaWorkClass=findAppSLAWorkClass(getWorkClassName(protocol))
        if (slaWorkClass == None):
            wclassAttributes = [["matchAction","Default_TC"],
                                ["name",getWorkClassName(protocol)],
                                ["type",getProtocolForWorkClass(protocol)],
                                ["description","This is a default workclass."]]
            slaWorkClass=AdminConfig.create("WorkClass",appLongName,wclassAttributes)
            print ("Created... default service workclass: "+getWorkClassName(protocol))

        if (slaWorkClass != None):
            createDefaultModules(slaWorkClass, modules, protocol)
    else:
        print "The application specified does not exist in this WAS deployment."
        errorExit()


def createDefaultModules(wcid, modules, protocol):
    for modulename in modules:                        
        if (protocol == "HTTP" or protocol == "SOAP"):
            uriExpression="*"
        else:
            uriExpression="*:*"

        wcModuleAttributes = [ ["moduleName",modulename],
                               ["matchExpression",uriExpression],
                               ["id",getWorkClassName(protocol)+":!:"+appname+":!:"+modulename]]
        workClassModule=AdminConfig.create("WorkClassModule",wcid,wcModuleAttributes)
        print ("Adding... workclass module: "+modulename)


#
# Finds application routing workclass in WAS repository.
#
# @param aWCName  The name of the workclass to look up.
# @return name    The name of the workclass if it exists or None
#                 if it does not exists.
# 
def findAppRoutingWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name and wcName.find("/deployments/") == -1 and wcName.find("/"+appname+".ear/") > 0):
                return wcName
   return None

#
# Finds the application service policy workclass in WAS repository.
#
# @param aWCName   The name of the application service policy workclass to look up.
# @return name    The name of the application service policy workclass if it exists or 
#                None if it does not exists.
#
def findAppSLAWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name and wcName.find("/deployments/") > 0 and wcName.find("/"+appname+"/") > 0):
              return wcName
   return None

#
# Finds the enterprise application in WAS repository.
#
# @param aAppName   The name of the enterprise application to look up.
# @return name    The name of the enterprise application if it exists or 
#                None if it does not exists.
#
def findDeployment(aAppName):
   appName=AdminConfig.getid("/Deployment:"+aAppName+"/")

   if (appName == ""):
       return None
   else:
       return appName

def getProtocolForWorkClass(protocol):
   if (protocol == "SOAP"):
      protocolType = "SOAPWORKCLASS"
   elif (protocol == "IIOP"):
      protocolType = "IIOPWORKCLASS"
   elif (protocol == "JMS"):
      protocolType = "JMSWORKCLASS" 
   else:
      protocolType = "HTTPWORKCLASS"
   return protocolType

def getWorkClassName(protocol):
   return "Default_"+protocol+"_WC" 

#
# Gets the list of applications installed (public and private).
#
# @return A list of application names.
#
def getApplicationList():
    applications = ""
    deployments = AdminConfig.list('Deployment').split(lineSeparator)    
    
    for deployment in deployments:
        appname = deployment.split("(")[0]        
        appname = appname.replace("\"","")

        if (not (appname in applications.split(","))):
            if (applications == ""):
                applications = appname
            else:
                applications = applications + ","+appname
        
    return applications.split(",")

#==========================================================================
# Begin main execution
#==========================================================================
appList = getApplicationList()
userID = MonitorUtil.generateUserID()
wksp = MonitorUtil.getWorkSpace(userID)
cellName = AdminConfig.list("Cell").split("(")[0]
print "Cell name="+cellName
print "Number of Applications to process="+str(len(appList))
appname = ""

for app in appList:
    #Do a check to see if the application is in need of migration
    print "Processing... application "+app
    appname = app
    
    webModules = ApplicationUtil.listWebModuleNames(cellName, appname, wksp)
    if len(webModules) > 0:
        if (findAppSLAWorkClass("Default_HTTP_WC") == None):
            print ("Creating... default HTTP service workclass")
            createAppSLAWorkClass(webModules, "HTTP")
        if (findAppRoutingWorkClass("Default_HTTP_WC") == None):
            print ("Creating... default HTTP routing workclass")
            createAppRoutingWorkClass(webModules, "HTTP")            

    soapModules = ApplicationUtil.listWebServiceModuleNames(cellName, appname, wksp)
    if len(soapModules) > 0:
        if (findAppSLAWorkClass("Default_SOAP_WC") == None):
            print ("Creating... default SOAP service workclass")
            createAppSLAWorkClass(soapModules, "SOAP")
        if (findAppRoutingWorkClass("Default_SOAP_WC") == None):
            print ("Creating... default SOAP routing workclass")
            createAppRoutingWorkClass(soapModules, "SOAP")
                    
    if (not ApplicationUtil.isAppDeployedOnZOS(appname, cellName)):
        iiopModules = ApplicationUtil.listEJBModuleNames(cellName, appname, wksp)
        if len(iiopModules) > 0:
            if (findAppSLAWorkClass("Default_IIOP_WC") == None):
                print ("Creating... default IIOP service workclass")
                createAppSLAWorkClass(iiopModules, "IIOP")
            
        jmsModules = ApplicationUtil.listJMSModuleNames(cellName, appname, wksp)
        if len(jmsModules) > 0:
            if (findAppSLAWorkClass("Default_JMS_WC") == None):
                print ("Creating... default JMS service workclass")
                createAppSLAWorkClass(jmsModules, "JMS")
            
print "Saving default workclass changes... "
AdminConfig.save()
print "Saving... completed."
MonitorUtil.removeWorkSpace(userID)